/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import javax.swing.*;
import javax.swing.table.*;

/**
   ConfigTableModel is the back-end behind MARS' configuration facility.
   It allows editing of the services enabled on a host. (Other configuration
   options, such as adding and deleting hosts, are handled by the config 
   user interface itself)
*/

// Revision History:
// Revision 0.1 18 Feb 1999 bht - initial entry

public class ConfigTableModel extends AbstractTableModel {

    /** Compile-time debugging switch. Should be false. */
    public static final boolean DEBUG = false;
    
    // Instance variables
    String[] headers;
    MARSConfig thisConfig;
    int SPOTS_POS;                        // a simulated constant...


    // Constants
    final static int HOST_POS = 0;
    
    /**
       Create a new ConfigTableModel on a specified MARSConfig.
       DO NOT USE MARSConfig.getCurrent() on this one, or the cancel feature
       won't work. Use a clone of getCurrent() instead.
    
       @param cloneConfig the configuration to muck about with.
       @return a new ConfigTableModel that will configure its MARSConfig
    */

    public ConfigTableModel (MARSConfig cloneConfig) {
	int i;
	
	// Dynamically build the column headers array
	headers = new String[2 + MARS.SVC_COUNT];   // host, services, SPOTS
	headers[0] = "Host";                        // the hostname is first
	for (i = 1; i < (MARS.SVC_COUNT + 1); i++)  // then services
	    headers[i] = MARS.SVC[i-1];
	headers[i] = "SPOTS";                       // then SPOTS
	SPOTS_POS = i;                              // store SPOTS position
	                                            // for later
	// set the config we're working with
	thisConfig = cloneConfig;
    }
    
    /**
       Return the number of columns (always constant)

       @return the number of columns
    */
    public int getColumnCount() {
	return headers.length;
    }
   
    /**
       Return the number of rows (equal to the number of hosts we're watching)

       @return the number of rows
    */
    public int getRowCount() {
	return thisConfig.getHostCount();
    }

    /**
       Return the name of the ith column

       @param col the column whose name to get
       @return the name of the specified column
    */

    public String getColumnName(int col) {
	return headers[col];
    }
    
    /**
       Return whether a given cell is editable.
       
       @param row the row of the cell we're checking the editability of
       @param col the column of the cell we're checking the editability of
       @return true if the cell is editable
    */
    public boolean isCellEditable(int row, int col) {
	if ((col > HOST_POS) && (col <= SPOTS_POS))
	    return true;
	else
	    return false;
    }

    /** 
	Return the Class of a given column.

	@param col the column index to return a class for.
	@return the Class of the specified column.
    */
    
    public Class getColumnClass (int col) {
	Class retval = null;
	
	if (col == HOST_POS) {
	    try {retval = Class.forName("java.lang.String");} 
	    catch (Exception ex) { System.out.println(ex);} }
	else if (col <= SPOTS_POS) {
	    try {retval = Class.forName("java.lang.Boolean");} 
	    catch (Exception ex) { System.out.println(ex); } }
	else
	    throw new IllegalArgumentException("Bad column "+col+" while determining column editor");
    
	return retval;
    }

    /**
       Return the value at a specified location in the table

       @return the value to be displayed at the specified location
       @param row the row of the cell getting the value of
       @param col the column of the cell we're getting the value of
    */

    public Object getValueAt(int row, int col) {
	Object retval;
	boolean probeEnabled;

	// First, get the host we're talking to
	Host thisHost = thisConfig.getHost(row);

	// Then select the information appropriate for the column
	// host - return the host name
	if (col == HOST_POS) {
	    retval = thisHost.getVisibleName();
	}
	// spots, get spots query not null
	else if (col == SPOTS_POS) {
	    probeEnabled = (thisHost.getSPOTSQuery() != null);
	    retval = new Boolean(probeEnabled);
	}
	// service, get numbered probe not null
	else if (col > HOST_POS && col < SPOTS_POS) {
	    probeEnabled = (thisHost.getSvcProbe(col - 1) != null);
	    retval = new Boolean(probeEnabled);
	}
	else
	    throw new IllegalArgumentException("Bad cell ("+row+", "+col+") while configuring host: "+thisHost.toString());

	return retval;
    }

    /**
       Set the value at a specified location in the table

       @param value the value to set the specified location to
       @param row the row of the cell getting the value of
       @param col the column of the cell we're getting the value of
    */

    public void setValueAt(Object value, int row, int col) {
	// First, get the host we're talking to
	Host thisHost = thisConfig.getHost(row);

	if (col > HOST_POS && col < SPOTS_POS)
	    thisHost.setSvcEnable(col - 1, ((Boolean)value).booleanValue());
	else if (col == SPOTS_POS) 
	    thisHost.setSPOTSEnable(((Boolean)value).booleanValue());
	else
	    throw new IllegalArgumentException("Bad cell ("+row+", "+col+") while configuring host: "+thisHost.toString());    
    }
}	    




