/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import javax.swing.*;

/**
   MARS is the entry-point for the MARS client application.
   main() uses static methods to initialize MARS piece by piece, while
   shutdown() saves the configuration and exits.
*/

public class MARS {
  
    /*
      Configuration section. Of course, some code needs to be added to the
      system to make it understand new services, but the service matrix,
      as it were, is here.
    */

    /** Release identification string */
    public static final String VERSION = "1.4";

    /** Number of services MARS knows about */
    public static final int SVC_COUNT = 6;

    /** Name of each service MARS knows about */
    public static final String SVC[] = 
                               {"SMTP","IMAP","POP","HTTP","FTP","SSH"};
    
    /** Service number of the SMTP service */
    public static final int SVC_SMTP = 0;
    
    /** Service number of the IMAP service */
    public static final int SVC_IMAP = 1;

    /** Service number of the POP service */
    public static final int SVC_POP = 2;
    
    /** Service number of the HTTP service */
    public static final int SVC_HTTP = 3;
    
    /** Service number of the FTP service */
    public static final int SVC_FTP = 4;
    
    /** Service number of the SSH service */
    public static final int SVC_SSH = 5;

    /** Number of system parameters SPOTS knows about */
    public static final int SPOT_COUNT = 3;
    
    /** Name of each system parameter SPOTS knows about */
    public static final String SPOT[] = {"Load",
					 "Disk",
                                         "Mem"};
    
    /** Parameter number of load average */
    public static final int SPOT_LOAD = 0;

    /** Parameter number of free disk */
    public static final int SPOT_DISK = 1;
    
    /** Parameter number of free memory */
    public static final int SPOT_MEM = 2;
    
    /**
       "Bootstrap" for MARS. Uses private static methods to start each
       MARS subsystem.

       @param args Program command line. Currently ignored.
    */

    public static final void main (String[] args) {
	printGPL();               // Display our GPL status
	initConfig();             // Read or create a MARSConfig
	initProbes();             // Create ProbeThreads for each Host
	initUI();                 // Create a user interface
	startThreads();           // Start the ProbeThreads and UI Thread.
    }

    /**
       Prints a notice on stdout for copyright and GPL purposes, pursuant
       to section 2(c) of the GNU General Public License, Version 2.
       Someday, this will be a graphical splash screen.
    */
    private static final void printGPL() {
	/*	System.out.println(
	 "MARS "+MARS.VERSION+" (c) 1999 Brian H. Trammell and Ben Parrott.");
	System.out.println(
	 "MARS is released without any warranty, and may be copied or");
	System.out.println(
	 "redistributed under the terms of the GNU General Public License,");
	System.out.println(
	 "available at http://www.gnu.org/copyleft/gpl.html");
	*/
    
	MARSWelcome myWelcome = new MARSWelcome();
	myWelcome.displayWelcome();
	myWelcome.dispose();
    }

    /**
       Gets a MARSConfig to hold our configuration, either from disk
       or by creating a new, empty host table.
    */

    private static final void initConfig() {
	// Try reading the MARS configuration file
	if (!(MARSConfig.readConfigFile())) {
	    // If that fails, notify the user on the console and 
	    // create a new MARSConfig object.
	    System.err.println
		("WARNING: MARS could not find its configuration file.");
	    System.err.println
		("\tCreating a new empty host table...");
	    new MARSConfig().makeCurrent();
	}
    }

    /**
       Creates a ProbeThread for each Host found in the host table.
    */

    public static final void initProbes() {
	int i;                           // for each host index
	
	for (i = 0; i < MARSConfig.getCurrent().getHostCount(); i++)
	    new ProbeThread(MARSConfig.getCurrent().getHost(i));
    }

    /**
       Sets up the user interface by instantiating MARSMainUI
    */

    private static final void initUI() {
	new MARSMainUI().show();
    }

    /**
       Makes MARS go by starting all the threads.
    */
    
    public static final void startThreads() {
	// tell the user we did this in the status bar
	MARSMainUI.getCurrent().setStatus("MARS is starting...");
	// start the probe threads.
	ProbeThread.startAll();
    }

    /**
       Makes MARS stop by stopping all the threads
    */

    public static final void stopThreads() {
	// stop the probe threads
	ProbeThread.stopAll();
	// tell the user we did this in the status bar
	MARSMainUI.getCurrent().setStatus("MARS is stopped.");
    }

    /**
       Shuts MARS down by stopping the threads, writing out the MARS config 
       file and exiting
    */
    
    public static final void shutdown() {
	// stop the threads
	stopThreads();
	// now write the config file
	MARSConfig.writeConfigFile();
	// now die!
	System.exit(0);
    }
}



