/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import org.altara.netlib.*;

// Revision History
// 1.4.7 bht 19 May 1999 - added sendDiskAlert method

/**
   This class contains static methods for sending problem notifications.
   Currently it contains email notification methods only.
*/

public class MARSNotifier {
    
    /**
       Send an email message notifying the recipient that the specified
       service on the specified host is down.

       @param service Name of the failing service
       @param host Name of the host the service is failing on
       @return true if the message was sent successfully
    */

    public static boolean sendServiceDown(String service, String host){
	
	SMTPMessage thisMessage;
	boolean msgStatus;
	
	// Create a new SMTPMessage, and set up the envelope and headers
	thisMessage=new SMTPMessage(MARSConfig.getCurrent().getNotifyServer(),
				    MARSConfig.getCurrent().getNotifyAddr(),
				    MARSConfig.getCurrent().getNotifyAddr(),
				    service + " down on " + host);

	// Fill out the body of the message
	thisMessage.addLine("MARS "+MARS.VERSION+" service status alert:");
	thisMessage.addLine("The "+service+" service is down on "+ host);
	
	// Try sending the message
	try {
	    msgStatus = thisMessage.sendMessage();
	} catch (Exception ex) {
	    msgStatus = false;
	}
	
	return msgStatus;
    }

    /**
       Send an email message notifying the recipient that the specified
       parameter is out of bounds on the specified host.

       @param parameter the SPOTS parameter we're warning about
       @param host the host the parameter is out of bounds on
       @param value the offending value
       @return true if the message was sent successfully
    */

    public static boolean sendParameterAlert(String parameter,
					     String host,
					     String value) {

	SMTPMessage thisMessage;
	boolean msgStatus;
	
	// Create a new SMTPMessage, and set up the envelope and headers
	thisMessage=new SMTPMessage(MARSConfig.getCurrent().getNotifyServer(),
				    MARSConfig.getCurrent().getNotifyAddr(),
				    MARSConfig.getCurrent().getNotifyAddr(),
				    parameter + " alert on " + host);

	// Fill out the body of the message
	thisMessage.addLine("MARS "+MARS.VERSION+" parameter alert:");
	thisMessage.addLine(parameter+" of "+ value +
			    " is out of bounds on "+ host);

	// Try sending the message
	try {
	    msgStatus = thisMessage.sendMessage();
	} catch (Exception ex) {
	    msgStatus = false;
	}
	
	return msgStatus;
    }
	
    /**
       Send an email message notifying the recipient that the specified
       filesystems on the specified host are low on disk space

       @param host the host disk space is low on
       @param fsList a space-delimited list of filesystems low on space
       @return true if the message was sent successfully
    */

    public static boolean sendDiskAlert(String host,
					String fsList) {

	SMTPMessage thisMessage;
	boolean msgStatus;
	
	// Create a new SMTPMessage, and set up the envelope and headers
	thisMessage=new SMTPMessage(MARSConfig.getCurrent().getNotifyServer(),
				    MARSConfig.getCurrent().getNotifyAddr(),
				    MARSConfig.getCurrent().getNotifyAddr(),
				    "Disk space alert on " + host);

	// Fill out the body of the message
	thisMessage.addLine("MARS "+MARS.VERSION+" disk space alert:");
	thisMessage.addLine("Disk space is low on the following filesystem(s)"+
			    " on "+ host+":");
	thisMessage.addLine(fsList);

	// Try sending the message
	try {
	    msgStatus = thisMessage.sendMessage();
	} catch (Exception ex) {
	    msgStatus = false;
	}
	
	return msgStatus;
    }
}
