/*
  Copyright 1998, 1999 Brian H. Trammell (brian@altara.org)

  This software (and its source code) are freely usable, modifiable,
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html.

  This software is released without any warranty whatsoever.

  Note this library software is NOT distributed under the GNU LGPL; if you 
  link an application with this software, it is ILLEGAL to distribute it
  under any terms incompatible with the GNU GPL.
*/

package org.altara.netlib;

import java.net.*;
import java.io.*;


/**
   ClientEngine is netlib's network "engine". Upon creation, it opens a 
   TCP connection to a remote machine. It sends any strings given it via its
   send() method, and gives any data it recieves to a Client's recieve()
   method.
*/


public class ClientEngine extends Thread {
  
  // Instance Variables
  Client             client;            // Client "parent" object
  Socket             clientSock;        // Network socket to server
  BufferedReader     sockIn;            // Socket input reader
  BufferedWriter     sockOut;           // Socket output writer
  String             host;
  int                port;

  // Constants
  /** Amount of time to wait between read attempts. */
  public static final long TICK = 10;      
  /** Compile-time debugging flag. Should be false. */
  public final static boolean DEBUG = false;
  
  // Constructors
  
  /**
   * Default constructor:
   * 
   * Create a new ClientEngine to connect to a specific host/port.
   *
   * @param client Client object to send recieved data to 
   * @param host hostname of server to connect to
   * @param port port number to connect to on server
   * @exception UnknownHostException if the host is unknwown
   * @exception IOException for other socket creation errors
   */
  
  public ClientEngine (Client client, String host, int port) 
    {
      this.client = client;
      this.host = host;
      this.port = port;
    }

  /**
     Opens the connection and starts the receiver thread.
  */

  public void connect() throws UnknownHostException, IOException {
    // Create a socket and associated readers/writers.
    clientSock = new Socket(host,port);
    if (DEBUG) System.out.println("cej: opened socket: "+clientSock);
    
    sockIn = new BufferedReader(new InputStreamReader(clientSock.getInputStream()));
    if (DEBUG) System.out.println("cej: created sockIn: "+sockIn);
    
    sockOut = new BufferedWriter (new OutputStreamWriter(clientSock.getOutputStream()));
    if (DEBUG) System.out.println("cej: created sockOut: "+sockOut);
    
    // Start running the ClientEngine processing thread
    this.start();
    if (DEBUG) System.out.println("cej: started ClientEngine thread.");
  }

      
  /**
   * send(data):
   *
   * Send a string to the server
   *
   * @param data String to send to the server
   * @exception IOException if data cannot be written to the socket
   */

  public void send (String data)
    throws IOException
    {
      sockOut.write(data,0,data.length());
      sockOut.flush();
      if (DEBUG) System.out.println("cej: sent: "+data);
    }

  /**
   * run():
   *
   * Poll for input over the socket. Call the client's receive() method
   * when each line is recieved.
   */

  public void run ()
    {
      // Run an infinite loop
      try {
	while (true) {
	  if (sockIn.ready()) { 
	    if (DEBUG) System.out.println("cej: sockIn ready");
	    client.receive(sockIn.readLine()); // Shovel a line to client
	    if (DEBUG) System.out.println("cej: called client.receive()");
	  }
	  try {
	    Thread.sleep(TICK);
	  } catch (InterruptedException ex) {
	    System.err.println("cej: WARNING: run() interrupted.");
	  }
	}  
      } catch (IOException ex) {
	if (DEBUG) System.out.println("cej: IOException in run()");
	client.socketDied();
	this.close();
      }
      try {
	if (DEBUG) System.out.println("cej: sleeping for "+TICK+" ms");
	Thread.sleep(TICK);
      } catch (InterruptedException ex) {
	System.err.println("cej: WARNING: run() interrupted.");
      }
    }

  /**
   * close():
   *
   * Close the engine's socket and terminate the thread.
   */

  public void close() {
    try {
      if (DEBUG) System.out.println("cej: close()");
      this.stop();
      if (DEBUG) System.out.println("cej: thread stopped");
      clientSock.close();
      if (DEBUG) System.out.println("cej: socket closed");
    } catch (IOException ex) {
      System.err.println("cej: WARNING: Socket already closed.");
    }
  }

  protected void finalize() {
    close();
  }
}






